/*
 * Copyright 2005 Red Hat, Inc.
 */
#include "../config.h"
#include <sys/types.h>
#include <errno.h>
#include <pwd.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <security/pam_modules.h>
#include "common.h"
#include "oddjob_dbus.h"

#ident "$Id: pam_oddjob_mkhomedir.c,v 1.1 2005/04/25 18:58:27 nalin Exp $"

static void
send_pam_oddjob_mkhomedir_request(pam_handle_t *pamh)
{
	const char *user;
	char *buf;
	size_t bufsize;
	struct passwd pwd, *pw;
	int ret;

	user = NULL;
	if ((pam_get_user(pamh, &user, "login: ") == PAM_SUCCESS) &&
	    (user != NULL) &&
	    (strlen(user) > 0)) {
		/* Attempt to look up information about the user. */
		bufsize = 4;
		do {
			pw = NULL;
			buf = malloc(bufsize);
			if (buf == NULL) {
				break;
			}
			ret = getpwnam_r(user, &pwd, buf, bufsize, &pw);
			if ((ret != 0) || (pw != &pwd)) {
				pw = NULL;
				free(buf);
				buf = NULL;
				if (errno == ERANGE) {
					bufsize += 4;
				} else {
					break;
				}
			}
		} while ((ret != 0) && (errno == ERANGE));
		if (pw != NULL) {
			/* If we're running with the user's privileges, then
			 * call the "mkmyhomedir" method. */
			if ((getuid() == pw->pw_uid) &&
			    (geteuid() == pw->pw_uid) &&
			    (getgid() == pw->pw_gid) &&
			    (getegid() == pw->pw_gid)) {
				ret = oddjob_dbus_call_method(ODDJOB_SERVICE_NAME,
							      ODDJOB_OBJECT_PATH,
							      ODDJOB_INTERFACE_NAME,
							      "mkmyhomedir",
							      NULL, 0,
							      NULL);
			} else {
				/* Otherwise, attempt to have a directory
				 * created for the user by calling the
				 * "mkhomedirfor" method. */
				ret = oddjob_dbus_call_method(ODDJOB_OBJECT_PATH,
							      ODDJOB_INTERFACE_NAME,
							      "mkhomedirfor",
							      user,
							      NULL, 0,
							      NULL);
			}
		}
		if (buf != NULL) {
			free(buf);
		}
	}
}

int
pam_sm_open_session(pam_handle_t *pamh, int flags, int argc, const char **argv)
{
	send_pam_oddjob_mkhomedir_request(pamh);
	return PAM_IGNORE;
}
int
pam_sm_close_session(pam_handle_t *pamh, int flags, int argc, const char **argv)
{
	return PAM_IGNORE;
}

int
pam_sm_acct_mgmt(pam_handle_t *pamh, int flags, int argc, const char **argv)
{
	send_pam_oddjob_mkhomedir_request(pamh);
	return PAM_IGNORE;
}
