/*
 * Copyright 2005 Red Hat, Inc.
 */
#include "../config.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <dbus/dbus.h>
#include "selinux.h"

#ident "$Id: selinux.c,v 1.1 2005/04/25 18:58:27 nalin Exp $"

#ifdef SELINUX

#include <selinux/selinux.h>

static dbus_bool_t
oddjob_check_selinux_enabled(void)
{
	static int selinux_enabled = -1;
	if (selinux_enabled == -1) {
		selinux_enabled = is_selinux_enabled();
	}
	return (selinux_enabled == 1);
}

void
oddjob_set_selinux_file_creation_context(const char *path, mode_t mode)
{
	security_context_t context;

	if (!oddjob_check_selinux_enabled()) {
		return;
	}

	context = NULL;
	if (matchpathcon(path, mode, &context) != 0) {
		if (context != NULL) {
			if (strcmp(context, "<<none>>") == 0) {
				oddjob_unset_selinux_file_creation_context();
			} else {
				setfscreatecon(context);
			}
			freecon(context);
		} else {
			oddjob_unset_selinux_file_creation_context();
		}
	}
}

void
oddjob_unset_selinux_file_creation_context(void)
{
	if (!oddjob_check_selinux_enabled()) {
		return;
	}
	setfscreatecon(NULL);
}

#else

void
oddjob_set_selinux_file_creation_context(const char *path, mode_t mode)
{
}

void
oddjob_unset_selinux_file_creation_context(void)
{
}

#endif
